@@ -216,7 +216,7 @@ static int video_output_init(struct video_output_ctx *output_ctx,
 	 */
 	if (image_format == AM7XXX_IMAGE_FORMAT_NV12) {
 		fprintf(stdout, "using raw output format\n");
-		output_codec_ctx->pix_fmt    = PIX_FMT_NV12;
+		output_codec_ctx->pix_fmt    = AV_PIX_FMT_NV12;
 		output_ctx->codec_ctx = output_codec_ctx;
 		output_ctx->raw_output = 1;
 		ret = 0;
@@ -235,8 +235,8 @@ static int video_output_init(struct video_output_ctx *output_ctx,
 	 * in particular they won't be 0, this is needed because they are used
 	 * as divisor somewhere in the encoding process */
 	output_codec_ctx->qmin       = output_codec_ctx->qmax = ((100 - (quality - 1)) * FF_QUALITY_SCALE) / 100;
-	output_codec_ctx->mb_lmin    = output_codec_ctx->lmin = output_codec_ctx->qmin * FF_QP2LAMBDA;
-	output_codec_ctx->mb_lmax    = output_codec_ctx->lmax = output_codec_ctx->qmax * FF_QP2LAMBDA;
+	output_codec_ctx->mb_lmin    = output_codec_ctx->qmin * FF_QP2LAMBDA;
+	output_codec_ctx->mb_lmax    = output_codec_ctx->qmax * FF_QP2LAMBDA;
 	output_codec_ctx->flags      |= CODEC_FLAG_QSCALE;
 	output_codec_ctx->global_quality = output_codec_ctx->qmin * FF_QP2LAMBDA;
 
@@ -276,7 +276,8 @@ static int am7xxx_play(const char *input_format_string,
 		       unsigned int upscale,
 		       unsigned int quality,
 		       am7xxx_image_format image_format,
-		       am7xxx_device *dev)
+		       am7xxx_device *dev,
+		       int dump_frame)
 {
 	struct video_input_ctx input_ctx;
 	struct video_output_ctx output_ctx;
@@ -320,6 +321,9 @@ static int am7xxx_play(const char *input_format_string,
 		ret = -ENOMEM;
 		goto cleanup_picture_raw;
 	}
+	picture_scaled->format = (output_ctx.codec_ctx)->pix_fmt;
+	picture_scaled->width = (output_ctx.codec_ctx)->width;
+	picture_scaled->height = (output_ctx.codec_ctx)->height;
 
 	/* calculate the bytes needed for the output image and create buffer for the output image */
 	out_buf_size = avpicture_get_size((output_ctx.codec_ctx)->pix_fmt,
@@ -354,6 +358,7 @@ static int am7xxx_play(const char *input_format_string,
 		goto cleanup_out_buf;
 	}
 
+	got_packet = 0;
 	while (run) {
 		/* read packet */
 		ret = av_read_frame(input_ctx.format_ctx, &in_packet);
@@ -381,11 +386,11 @@ static int am7xxx_play(const char *input_format_string,
 			goto end_while;
 		}
 
-		/* if we get the complete frame */
+		/* if we got the complete frame */
 		if (got_picture) {
 			/* convert it to YUV */
 			sws_scale(sw_scale_ctx,
-				  (const uint8_t * const*)picture_raw->data,
+				  (const uint8_t * const *)picture_raw->data,
 				  picture_raw->linesize,
 				  0,
 				  (input_ctx.codec_ctx)->height,
@@ -416,15 +421,19 @@ static int am7xxx_play(const char *input_format_string,
 			}
 
 #ifdef DEBUG
-			char filename[NAME_MAX];
-			FILE *file;
-			if (!output_ctx.raw_output)
-				snprintf(filename, NAME_MAX, "out_q%03d.jpg", quality);
-			else
-				snprintf(filename, NAME_MAX, "out.raw");
-			file = fopen(filename, "wb");
-			fwrite(out_picture, 1, out_picture_size, file);
-			fclose(file);
+			if (dump_frame) {
+				char filename[NAME_MAX];
+				FILE *file;
+				if (!output_ctx.raw_output)
+					snprintf(filename, NAME_MAX, "out_q%03d.jpg", quality);
+				else
+					snprintf(filename, NAME_MAX, "out.raw");
+				file = fopen(filename, "wb");
+				fwrite(out_picture, 1, out_picture_size, file);
+				fclose(file);
+			}
+#else
+			(void) dump_frame;
 #endif
 
 			ret = am7xxx_send_image_async(dev,
@@ -434,7 +443,7 @@ static int am7xxx_play(const char *input_format_string,
 						      out_picture,
 						      out_picture_size);
 			if (ret < 0) {
-				perror("am7xxx_send_image");
+				perror("am7xxx_send_image_async");
 				run = 0;
 				goto end_while;
 			}
@@ -593,6 +602,9 @@ static void usage(char *name)
 	printf("usage: %s [OPTIONS]\n\n", name);
 	printf("OPTIONS:\n");
 	printf("\t-d <index>\t\tthe device index (default is 0)\n");
+#ifdef DEBUG
+	printf("\t-D \t\t\tdump the last frame to a file (only active in DEBUG mode)\n");
+#endif
 	printf("\t-f <input format>\tthe input device format\n");
 	printf("\t-i <input path>\t\tthe input path\n");
 	printf("\t-o <options>\t\ta comma separated list of input format options\n");
@@ -640,8 +652,9 @@ int main(int argc, char *argv[])
 	int format = AM7XXX_IMAGE_FORMAT_JPEG;
 	am7xxx_context *ctx;
 	am7xxx_device *dev;
+	int dump_frame = 0;
 
-	while ((opt = getopt(argc, argv, "d:f:i:o:s:uF:q:l:p:z:h")) != -1) {
+	while ((opt = getopt(argc, argv, "d:Df:i:o:s:uF:q:l:p:z:h")) != -1) {
 		switch (opt) {
 		case 'd':
 			device_index = atoi(optarg);
@@ -651,6 +664,12 @@ int main(int argc, char *argv[])
 				goto out;
 			}
 			break;
+		case 'D':
+			dump_frame = 1;
+#ifndef DEBUG
+			fprintf(stderr, "Warning: the -D option is only active in DEBUG mode.\n");
+#endif
+			break;
 		case 'f':
 			input_format_string = strdup(optarg);
 			break;
@@ -849,7 +868,8 @@ int main(int argc, char *argv[])
 			  upscale,
 			  quality,
 			  format,
-			  dev);
+			  dev,
+			  dump_frame);
 	if (ret < 0) {
 		fprintf(stderr, "am7xxx_play failed\n");
 		goto cleanup;
